@@ -51,6 +51,13 @@ impl<const N:usize> tstr<N>
       }
    }//make
 
+   /// version of make that does not truncate
+   pub fn try_make(s:&str) -> Result<tstr<N>,&str>
+   {
+       if s.len()>N-1 {Err(s)}
+       else {Ok(tstr::make(s))}
+   }
+
    /// creates an empty string, equivalent to tstr::default()
    pub fn new() -> tstr<N>
    {
@@ -86,13 +93,16 @@ impl<const N:usize> tstr<N>
       &self.chrs[1..self.len()+1]
    }
 
-   /// converts tstr to &str using [std::str::from_utf8]
+   /// converts tstr to &str using [std::str::from_utf8_unchecked]
    pub fn to_str(&self) -> &str
+   {
+      unsafe {std::str::from_utf8_unchecked(&self.chrs[1..self.len()+1])}
+   }
+   /// alias for [tstr::to_str]
+   pub fn as_str(&self) -> &str
    {
       std::str::from_utf8(&self.chrs[1..self.len()+1]).unwrap()
    }
-   /// alias for [tstr::to_str]
-   pub fn as_str(&self) -> &str {self.to_str()}
 
    /// changes a character at character position i to c.  This function
    /// requires that c is in the same character class (ascii or unicode)
@@ -175,40 +185,27 @@ impl<const N:usize> tstr<N>
    { self.to_str().char_indices() }
 }//impl tstr<N>
 
-
-impl<const N:usize> std::convert::From<&str> for tstr<N>
+impl<const N:usize> std::convert::AsRef<str> for tstr<N>
 {
-  /// creates a new tstr<N> with given &str.  If the length of s exceeds
-  /// N, the extra characters are ignored.
-  fn from(s:&str) -> tstr<N>
+  fn as_ref(&self) -> &str { self.to_str() }
+}
+impl<const N:usize> std::convert::AsMut<str> for tstr<N>
+{
+  fn as_mut(&mut self) -> &mut str
   {
-     tstr::make(s)
+      let blen = self.len()+1;
+      unsafe {
+        std::str::from_utf8_unchecked_mut(&mut self.chrs[1..blen])
+      }
   }
 }
-
-impl<const N:usize> std::convert::From<&mut str> for tstr<N>
+impl<T:AsRef<str>+?Sized, const N:usize> std::convert::From<&T> for tstr<N>
 {
-  /// creates a new tstr<N> with given &str.  If the length of s exceeds
-  /// N, the extra characters are ignored.
-  fn from(s:&mut str) -> tstr<N>
-  {
-     tstr::make(s)
-  }
+   fn from(s:&T) -> tstr<N> { tstr::make(s.as_ref()) }
 }
-
-impl<const N:usize> std::convert::From<&String> for tstr<N>
+impl<T:AsMut<str>+?Sized, const N:usize> std::convert::From<&mut T> for tstr<N>
 {
-  fn from(s:&String) -> tstr<N>
-  {
-     tstr::<N>::make(&s[..])
-  }
-}
-impl<const N:usize> std::convert::From<&mut String> for tstr<N>
-{
-  fn from(s:&mut String) -> tstr<N>
-  {
-     tstr::<N>::make(&s[..])
-  }
+   fn from(s:&mut T) -> tstr<N> { tstr::make(s.as_mut()) }
 }
 
 impl<const N:usize> std::convert::From<String> for tstr<N>
@@ -227,14 +224,6 @@ impl<const N:usize,const M:usize> std::convert::From<fstr<M>> for tstr<N>
   }
 }
 
-impl<const N:usize,const M:usize> std::convert::From<&fstr<M>> for tstr<N>
-{
-  fn from(s:&fstr<M>) -> tstr<N>
-  {
-     tstr::<N>::make(&s.to_str())
-  }
-}
-
 impl<const N:usize,const M:usize> std::convert::From<zstr<M>> for tstr<N>
 {
   fn from(s:zstr<M>) -> tstr<N>
@@ -243,13 +232,7 @@ impl<const N:usize,const M:usize> std::convert::From<zstr<M>> for tstr<N>
   }
 }
 
-impl<const N:usize,const M:usize> std::convert::From<&zstr<M>> for tstr<N>
-{
-  fn from(s:&zstr<M>) -> tstr<N>
-  {
-     tstr::<N>::make(&s.to_str())
-  }
-}
+
 
 impl<const N:usize> std::cmp::PartialOrd for tstr<N>
 {
@@ -336,11 +319,13 @@ impl<const N:usize,const M:usize> PartialEq<tstr<N>> for fstr<M>
   fn eq(&self, other:&tstr<N>) -> bool
   { other.to_str()==self.to_str() }
 }
+/*
 impl<const N:usize, const M:usize> PartialEq<&tstr<N>> for fstr<M>
 {
   fn eq(&self, other:&&tstr<N>) -> bool
   { other.to_str()==self.to_str() }
 }
+*/
 impl<const N:usize,const M:usize> PartialEq<fstr<N>> for tstr<M>
 {
   fn eq(&self, other:&fstr<N>) -> bool
