@@ -26,7 +26,8 @@ pub struct zstr<const N:usize>
 impl<const N:usize> zstr<N>
 {
   /// creates a new zstr<N> with given &str.  If the length of s exceeds
-  /// N, the extra characters are ignored.  This function is also called by
+  /// N, the extra characters are ignored and a warning sent to stderr.
+  /// This function is also called by
   /// several others including [zstr::from].  This function can now handle
   /// utf8 strings properly.
   pub fn make(s:&str) -> zstr<N>
@@ -51,6 +52,14 @@ impl<const N:usize> zstr<N>
       }
    }//make
 
+   /// version of make that does not truncate
+   pub fn try_make(s:&str) -> Result<zstr<N>,&str>
+   {
+       if s.len()>N-1 {Err(s)}
+       else {Ok(zstr::make(s))}
+   }
+
+
    /// creates an empty string, equivalent to zstr::default()
    pub fn new() -> zstr<N>
    {
@@ -89,13 +98,20 @@ impl<const N:usize> zstr<N>
       &self.chrs[..self.blen()]
    }
 
-   /// converts zstr to &str using [std::str::from_utf8]
+   /// converts zstr to &str using [std::str::from_utf8_unchecked].
+   /// Since zstr can only be constructed from valid utf8 sources,
+   /// this conversion is safe.
    pub fn to_str(&self) -> &str
    {
-      std::str::from_utf8(&self.chrs[0..self.blen()]).unwrap()
+        unsafe {
+          std::str::from_utf8_unchecked(&self.chrs[0..self.blen()])
+        }
+   }
+   /// checked version of [zstr::to_str]
+   pub fn as_str(&self) -> &str
+   {
+     std::str::from_utf8(&self.chrs[0..self.blen()]).unwrap()
    }
-   /// alias for [zstr::to_str]
-   pub fn as_str(&self) -> &str {self.to_str()}
 
    /// changes a character at character position i to c.  This function
    /// requires that c is in the same character class (ascii or unicode)
@@ -186,7 +202,31 @@ impl<const N:usize> zstr<N>
    { self.to_str().char_indices() }
 }//impl zstr<N>
 
+impl<const N:usize> std::convert::AsRef<str> for zstr<N>
+{
+  fn as_ref(&self) -> &str { self.to_str() }
+}
+impl<const N:usize> std::convert::AsMut<str> for zstr<N>
+{
+  fn as_mut(&mut self) -> &mut str
+  {
+      let blen = self.blen();
+      unsafe {
+        std::str::from_utf8_unchecked_mut(&mut self.chrs[0..blen])
+      }
+  }
+}
 
+impl<T:AsRef<str>+?Sized, const N:usize> std::convert::From<&T> for zstr<N>
+{
+   fn from(s:&T) -> zstr<N> { zstr::make(s.as_ref()) }
+}
+impl<T:AsMut<str>+?Sized, const N:usize> std::convert::From<&mut T> for zstr<N>
+{
+   fn from(s:&mut T) -> zstr<N> { zstr::make(s.as_mut()) }
+}
+
+/*
 impl<const N:usize> std::convert::From<&str> for zstr<N>
 {
   /// creates a new zstr<N> with given &str.  If the length of s exceeds
@@ -221,6 +261,23 @@ impl<const N:usize> std::convert::From<&mut String> for zstr<N>
      zstr::<N>::make(&s[..])
   }
 }
+impl<const N:usize,const M:usize> std::convert::From<&fstr<M>> for zstr<N>
+{
+  fn from(s:&fstr<M>) -> zstr<N>
+  {
+     zstr::<N>::make(&s.to_str())
+  }
+}
+
+impl<const N:usize,const M:usize> std::convert::From<&tstr<M>> for zstr<N>
+{
+  fn from(s:&tstr<M>) -> zstr<N>
+  {
+     zstr::<N>::make(&s.to_str())
+  }
+}
+
+*/
 
 impl<const N:usize> std::convert::From<String> for zstr<N>
 {
@@ -238,22 +295,6 @@ impl<const N:usize,const M:usize> std::convert::From<fstr<M>> for zstr<N>
   }
 }
 
-impl<const N:usize,const M:usize> std::convert::From<&fstr<M>> for zstr<N>
-{
-  fn from(s:&fstr<M>) -> zstr<N>
-  {
-     zstr::<N>::make(&s.to_str())
-  }
-}
-
-impl<const N:usize,const M:usize> std::convert::From<&tstr<M>> for zstr<N>
-{
-  fn from(s:&tstr<M>) -> zstr<N>
-  {
-     zstr::<N>::make(&s.to_str())
-  }
-}
-
 impl<const N:usize,const M:usize> std::convert::From<tstr<M>> for zstr<N>
 {
   fn from(s:tstr<M>) -> zstr<N>
@@ -282,7 +323,8 @@ impl<const N:usize> std::cmp::Ord for zstr<N>
 impl<const M:usize> zstr<M>
 {
   /// converts an zstr\<M\> to an zstr\<N\>. If the length of the string being
-  /// converted is greater than N, the extra characters will be ignored.
+  /// converted is greater than N, the extra characters are ignored and
+  /// a warning sent to stderr.
   /// This operation produces a copy (non-destructive).
   /// Example:
   ///```ignore
@@ -302,39 +344,6 @@ impl<const M:usize> zstr<M>
   }//resize
 }//impl zstr<M>
 
-/*  OUT
-/// [IntoIterator] struct for zstr
-pub struct zstriter<const N:usize>
-{
-   fs : zstr<N>,
-   len: usize, // char length
-   i : usize,
-}
-impl<const N:usize> Iterator for zstriter<N>
-{
-   type Item = char;
-   fn next(&mut self) -> Option<char>
-   {
-      if self.i<self.len {
-        self.i+=1;
-        self.to_str().chars().nth(self.i-1)
-      } else {None}
-   }
-}
-impl<const N:usize> IntoIterator for zstr<N>
-{
-  type Item = char;
-  type IntoIter = zstriter<N>;
-  fn into_iter(self) -> zstriter<N>
-  {
-     zstriter {
-       fs : self,
-       len: self.len(),
-       i : 0,
-     }
-  }
-}//IntoIterator
-*/
 
 
 impl<const N:usize> std::fmt::Display for zstr<N>
@@ -392,11 +401,13 @@ impl<const N:usize,const M:usize> PartialEq<zstr<N>> for fstr<M>
   fn eq(&self, other:&zstr<N>) -> bool
   { other.to_str()==self.to_str() }
 }
+/*
 impl<const N:usize, const M:usize> PartialEq<&zstr<N>> for fstr<M>
 {
   fn eq(&self, other:&&zstr<N>) -> bool
   { other.to_str()==self.to_str() }
 }
+*/
 impl<const N:usize,const M:usize> PartialEq<fstr<N>> for zstr<M>
 {
   fn eq(&self, other:&fstr<N>) -> bool
